<?php

namespace Netzmacher\Browser\Controller;

use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\DataHandling\DataHandler;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\Controller\ActionController;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2015 - 2018 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class for frontend editing
 *
 * @package TYPO3
 * @subpackage browser
 * @author Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @version 8.0.17
 * @since 7.4.0
 * @internal #i0215
 */
class FrontendEditingController extends ActionController
{

	/**
	 * @var string Extension key
	 */
	public $extKey = 'browser';

	/**
	 * @var array Extension Configuration from ext_conf_template.txt
	 */
	private $_aExtConf;

	/**
	 * @var array record for an INSERT or an UPDATE
	 */
	private $_aKeyValues = array();

	/**
	 * @var array Powermail GET and POST params
	 */
	private $_aPowermailGP;

	/**
	 * @var int Last inserted id
	 */
	private $_aTableSQLLastID;

	/**
	 * @var string allowed tables of the fields from the form
	 */
	private $_aTables;

	/**
	 * @var TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer
	 */
	private $_oCObj;

	/**
	 * @var Netzmacher\Browser\Utility\FrontendEditing\DRS
	 */
	private $_oDRS;

	/**
	 * @var TYPO3\CMS\Core\Imaging\GraphicalFunctions
	 */
	private $_oGraphFuncObj;

	/**
	 * @var object
	 */
	private $_oParamsPowermail;

	/**
	 * @var Netzmacher\Browser\Utility\FrontendEditing\Prefill
	 */
	private $_oPrefill;

	/**
	 * @var Netzmacher\Browser\Utility\FrontendEditing\TCA
	 */
	private $_oTCA;

	/**
	 * @var Netzmacher\Browser\Utility\FrontendEditing\Tables
	 */
	private $_oTables;

	/**
	 * @var string The absolute path of the current file
	 */
	private $_sCurrentFileAbsPath;

	/**
	 * @var string The path of the current file
	 */
	private $_sCurrentFilePath;

	/**
	 * @var string The current table
	 */
	private $_sCurrentTable;

	/**
	 * @var int The uid of the current table record
	 */
	private $_sCurrentTableRecordUid;

	/**
	 * @var string path to the lanugage file
	 */
	private $_sLanguageFilePath = 'Resources/Private/Language/FeEditing.xlf';

	/**
	 * @var string Path to powermail uploads. Without any ending slash
	 */
	private $_sPowermailPath = 'uploads/tx_powermail';

	/**
	 * @var string Default upload folder
	 */
	private $_sUploadfolder = 'fileadmin/user_upload';

//  /**
//   * @var boolean Flag for the DRS - Development Reporting System
//   */
//  protected $b_drs_all = FALSE;
//  protected $b_drs_error = FALSE;
//  protected $b_drs_warn = FALSE;
//  protected $b_drs_info = FALSE;
//  protected $b_drs_frontendediting = FALSE;

	/**
	 * addAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 8.0.18
	 * @since 8.0.18
	 * @internal #i0296
	 *
	 */
	public function addAction()
	{
		$this->_init();
		$this->_viewAssignAdd();
		if( !$this->_requirements( 'add' ) )
		{
			return;
		}
		$this->_sessionSet( 'add' );
		//var_dump(__METHOD__, __LINE__, $this->_sessionGet());
		switch( true )
		{
			case($this->_powermailActionIsConfirmation()):
//				$this->_tables();
				break;
			case($this->_powermailActionIsCreate()):
				$this->_tables();
//				$this->view->assignMultiple( array( 'promptHeader' => 'false' ) );
				$this->view->assignMultiple(
								array(
										'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelForward' ),
										'buttonCancelClass' => 'btn btn-primary button expand',
										'promptButtons' => 'true'
								)
				);
				$this->_sessionUnset();
				$this->_clearCache( );
				break;
			case($this->_powermailActionIsForm()):
			default:
				$this->_powermailPrefill();
				$this->view->assignMultiple( array( 'promptButtons' => 'true' ) );
				break;
		}
		//$this->_powermailUnsetParamAction();		
	}

	/**
	 * copyAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 8.0.18
	 * @since 8.0.18
	 * @internal #i0296
	 *
	 */
	public function copyAction()
	{
		$this->_init();
		$this->_viewAssignCopy();
		if( !$this->_requirements( 'copy' ) )
		{
			return;
		}
		$this->_sessionSet( 'copy' );
		//var_dump(__METHOD__, __LINE__, $this->_sessionGet());

		$this->_tables();
//				$this->view->assignMultiple( array( 'promptHeader' => 'false' ) );
		$this->view->assignMultiple(
						array(
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelForward' ),
								'buttonCancelClass' => 'btn btn-primary button expand',
								'class' => 'success',
								'prompt' => $this->_oPlugin->pi_getLL( 'actionCopyPromptSuccess' ),
								'promptButtons' => 'true'
						)
		);
		$this->_sessionUnset();
		$this->_clearCache(  );
	}

	/**
	 * deleteAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 8.0.18
	 * @since 8.0.18
	 * @internal #i0296
	 *
	 */
	public function deleteAction()
	{
		$this->_init();
		$this->_viewAssignDelete();
		if( !$this->_requirements( 'delete' ) )
		{
			return;
		}
	}

	/**
	 * deleterecordAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 8.0.18
	 * @since 8.0.18
	 * @internal #i0296
	 *
	 */
	public function deleterecordAction()
	{
		$this->_init();
		$this->_viewAssignDelete();
		if( !$this->_requirements( 'delete' ) )
		{
			return;
		}
		$this->_sessionSet( 'deleterecord' );
		//var_dump(__METHOD__, __LINE__, $this->_sessionGet());

		$this->_tables();
//				$this->view->assignMultiple( array( 'promptHeader' => 'false' ) );
		$this->view->assignMultiple(
						array(
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelForward' ),
								'buttonCancelClass' => 'btn btn-primary button expand',
								'class' => 'success',
								'prompt' => $this->_oPlugin->pi_getLL( 'actionDeletePromptSuccess' ),
								'promptButtons' => 'true'
						)
		);
		$this->_sessionUnset();
		$this->_clearCache(  );
	}

	/**
	 * getSqlInsertId( ) :
	 *
	 * @return int
	 * @access protected
	 * @version 8.0.21
	 * @since 8.0.18
	 */
	protected function getSqlInsertId( $table )
	{
		return ( int ) $this->_aTableSQLLastID[ $table ];
	}

	/**
	 * indexAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 8.0.18
	 * @since 8.0.18
	 *
	 */
	public function indexAction()
	{
		$this->_init();
		$action = $this->_getAction();
		//$this->_sessionSet();
		switch( true )
		{
			case($this->_powermailActionIsConfirmation()):
			case($this->_powermailActionIsCreate()):
			case($this->_powermailActionIsForm()):
				if( $action == 'add' )
				{
					$this->addAction();
				}
				if( $action == 'update' )
				{
					$this->updateAction();
				}
				break;
			default:
				if( $action == 'add' )
				{
					$this->addAction();
				}
				else
				{
					$this->_viewAssignIndex();
					if( !$this->_requirements( 'index' ) )
					{
						return;
					}
					$this->_powermailUnsetParamAction();
					$this->_sessionUnset();
				}
				break;
		}
	}

	/**
	 * updateAction( ) : 
	 *
	 * @return void
	 * @access public
	 * @version 8.0.18
	 * @since 8.0.18
	 * @internal #i0296
	 *
	 */
	public function updateAction()
	{
		$this->_init();
		$this->_viewAssignUpdate();
		if( !$this->_requirementsFeUser( 'update' ) )
		{
			return false;
		}
		$this->_sessionSet( 'update' );
		switch( true )
		{
			case($this->_powermailActionIsConfirmation()):
//				$this->_tables();
				break;
			case($this->_powermailActionIsCreate()):
				$this->_tables();
//				$this->view->assignMultiple( array( 'promptHeader' => 'false' ) );
				$this->view->assignMultiple(
								array(
										'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelForward' ),
										'buttonCancelClass' => 'btn btn-primary button expand',
										'promptButtons' => 'true'
								)
				);
				$this->_sessionUnset();
				$this->_clearCache(  );
				break;
			case($this->_powermailActionIsForm()):
				$this->_powermailPrefill();
				$this->view->assignMultiple( array( 'promptButtons' => 'true' ) );
				break;
			default:
				if( !$this->_requirements( 'update' ) )
				{
					return;
				}
				$this->_powermailPrefill();
				$this->view->assignMultiple( array( 'promptButtons' => 'true' ) );
				break;
		}
		//$this->_powermailUnsetParamAction();		
	}

	/**
	 * _DRSprompt( ) :
	 *
	 * @param string $prompt
	 * @param integer $line
	 * @param integer $type
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 *
	 */
	private function _DRSprompt( $prompt, $line = __LINE__, $type = 0 )
	{
		if( !$this->_oDRS->getDrsFrontendEditing() )
		{
			return;
		}
		GeneralUtility::devlog( '[INFO/FE] ' . $prompt, __CLASS__ . '#' . $line, $type );
	}

	/**
	 * _DRSpromptData( ) :
	 *
	 * @param array $data
	 * @param integer $line
	 * @param integer $type
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 *
	 */
	private function _DRSpromptData( $data, $line = __LINE__, $type = 0 )
	{
		if( !$this->_oDRS->getDrsFrontendEditing() )
		{
			return;
		}

		foreach( ( array ) $data AS $field => $value )
		{
			$aPrompt[] = $field . ': "' . $value . '"';
		}
		$prompt = implode( ', ', ( array ) $aPrompt );
		$prompt = 'Fields are added to cObj. Fields are available by TypoScript! ' . $prompt;
		$this->_DRSprompt( $prompt, $line, $type );
	}

	/**
	 * _DRSpromptData( ) :
	 *
	 * @param array $data
	 * @param integer $line
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 *
	 */
	private function _DRSswitchActionDie( $action, $method, $line )
	{
		$header = 'ERROR: undefined case';
		$text = 'Case can\'t handled. Case is "' . $action . '"';
		$this->_zzDieWiPrompt( $header, $text, $method, $line );
	}

	/**
	 * _SQLCopyCopy( ) :
	 *
	 * @param string $table
	 * @param string $where
	 * @param string $set
	 * @param integer $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _SQLCopy( $table, $where, $set, $line, $bPromptQueryOnly = 0 )
	{
		if( !$this->_SQLCopyHasRecords( $table, $where, $bPromptQueryOnly ) )
		{
			return;
		}
		$queries = array(
				'drop01' => 'DROP TABLE IF EXISTS tmp ; ',
				'create' => 'CREATE TABLE tmp SELECT * FROM ' . $table . ' WHERE ' . $where . '; ',
				'update' => 'UPDATE tmp SET ' . $set . '; ',
				'insert' => 'INSERT INTO ' . $table . ' SELECT * FROM tmp; ',
				'drop02' => 'DROP TABLE tmp ; ',
		);

		switch( true )
		{
			case($bPromptQueryOnly == 1):
				$this->_DRSprompt( 'Queries: ' . PHP_EOL . implode( PHP_EOL, $queries ), $line );
				$this->_DRSprompt( 'Query is for prompt only and will not executed.', $line );
				var_dump( __METHOD__, __LINE__, 'Queries: ' . PHP_EOL . implode( PHP_EOL, $queries ) );
				break;
			case(FALSE):
			default:
				$this->_DRSprompt( 'Queries: ' . PHP_EOL . implode( PHP_EOL, $queries ), $line );
				$GLOBALS[ 'TYPO3_DB' ]->sql_query( $queries[ 'drop01' ] );
				$this->_SQLError( $queries[ 'drop01' ], $line );
				$GLOBALS[ 'TYPO3_DB' ]->sql_query( $queries[ 'create' ] );
				$this->_SQLError( $queries[ 'create' ], $line );
				$GLOBALS[ 'TYPO3_DB' ]->sql_query( $queries[ 'update' ] );
				$this->_SQLError( $queries[ 'update' ], $line );
				$GLOBALS[ 'TYPO3_DB' ]->sql_query( $queries[ 'insert' ] );
				$this->_SQLError( $queries[ 'insert' ], $line );
				$this->_aTableSQLLastID[ $table ] = $GLOBALS[ 'TYPO3_DB' ]->sql_insert_id();
				$GLOBALS[ 'TYPO3_DB' ]->sql_query( $queries[ 'drop02' ] );
				$this->_SQLError( $queries[ 'drop02' ], $line );
				break;
		}
	}

	/**
	 * _SQLCopyHasRecords( ) :
	 *
	 * @param string $query
	 * @param boolean $bPromptQuery
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _SQLCopyHasRecords( $table, $where, $bPromptQuery = 0 )
	{
		$hasRecords = false;
		$query = 'SELECT count(*) FROM ' . $table . ' WHERE ' . $where . '; ';

		$res = $GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$row = $GLOBALS[ 'TYPO3_DB' ]->sql_fetch_row( $res );

		if( $row[ 0 ] > 0 )
		{
			$hasRecords = true;
		}

		if( !$bPromptQuery )
		{
			return $hasRecords;
		}
		var_dump( __METHOD__, __LINE__, $query, $row );

		return $hasRecords;
	}

	/**
	 * _SQLCopyMain( ) :
	 *
	 * @param string $table
	 * @param integer $uid
	 * @param integer $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _SQLCopyMain( $table, $uid, $line, $bPromptQueryOnly = 0 )
	{
		$params = null;
		if( isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ 'tstamp' ] ) )
		{
			$params = $params . ', tstamp = UNIX_TIMESTAMP()';
		}
		if( isset( $GLOBALS[ 'TCA' ][ $table ][ 'columns' ][ 'crdate' ] ) )
		{
			$params = $params . ', crdate = UNIX_TIMESTAMP()';
		}
		$where = 'uid = ' . $uid;
		$set = 'uid = null' . $params;
		$this->_SQLCopy( $table, $where, $set, $line, $bPromptQueryOnly );
	}

	/**
	 * _SQLCopyMM( ) :
	 *
	 * @param string $table
	 * @param string $set
	 * @param string $where
	 * @param integer $uidSrce
	 * @param integer $uidDest
	 * @param integer $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _SQLCopyMM( $table, $set, $where, $uidSrce, $uidDest, $line, $bPromptQueryOnly = 0 )
	{
		//var_dump( __METHOD__, __LINE__, 'Queries: ' . PHP_EOL . implode( PHP_EOL, $queries ) );
		$this->_SQLCopy( $table, $where, $set, $line, $bPromptQueryOnly );
	}

	/**
	 * _SQLDelete( ) :
	 *
	 * @param string $table
	 * @param string $where
	 * @param integer $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _SQLDelete( $table, $where, $line, $bPromptQueryOnly = 0 )
	{
		$query = $GLOBALS[ 'TYPO3_DB' ]->DELETEquery(
						$table
						, $where
		);

		switch( true )
		{
			case($bPromptQueryOnly == 1):
				$this->_DRSprompt( 'Query: ' . $query, $line );
				$this->_DRSprompt( 'Query is for prompt only and will not executed.', $line );
				var_dump( __METHOD__, __LINE__, $query );
				break;
			case(FALSE):
			default:
				$this->_DRSprompt( 'Query: ' . $query, $line );
				$GLOBALS[ 'TYPO3_DB' ]->exec_DELETEquery(
								$table
								, $where
				);
				$this->_SQLError( $query, $line );
				break;
		}
	}

	/**
	 * _SQLError( ) :
	 *
	 * @param string $query
	 * @param integer $line
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _SQLError( $query, $line = __LINE__ )
	{
		$sqlError = $GLOBALS[ 'TYPO3_DB' ]->sql_error();
		if( empty( $sqlError ) )
		{
			return;
		}

		$sqlErrno = $GLOBALS[ 'TYPO3_DB' ]->sql_errno();
		$header = 'SQL-ERROR';
		$text = ''
						. '<p>'
						. '  Error: ' . $sqlError . '<br />'
						. '  Error number: ' . $sqlErrno . '<br />'
						. '  Query: ' . $query . '<br />'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, $line );
	}

	/**
	 * _SQLInsert( ) :
	 *
	 * @param string $table
	 * @param integer $data
	 * @param integer $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _SQLInsert( $table, $data, $line, $bPromptQueryOnly = 0 )
	{
		$query = $GLOBALS[ 'TYPO3_DB' ]->INSERTquery(
						$table
						, $data
		);
		//var_dump( __METHOD__, __LINE__, $query, $data );
		switch( $bPromptQueryOnly )
		{
			case(TRUE):
				$this->_DRSprompt( 'Query: ' . $query, $line );
				$this->_DRSprompt( 'Query is for prompt only and will not executed.', $line );
				var_dump( __METHOD__, __LINE__, $query );
				break;
			case(FALSE):
			default:
				$this->_DRSprompt( 'Query: ' . $query, $line );
				$GLOBALS[ 'TYPO3_DB' ]->exec_INSERTquery(
								$table
								, $data
				);
				$this->_SQLError( $query, $line );
				$this->_aTableSQLLastID[ $table ] = $GLOBALS[ 'TYPO3_DB' ]->sql_insert_id();
				break;
		}
	}

	/**
	 * _SQLUpdate( ) :
	 *
	 * @param string $table
	 * @param integer $uid
	 * @param integer $data
	 * @param integer $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _SQLUpdate( $table, $uid, $data, $line, $bPromptQueryOnly = 0 )
	{
		$query = $GLOBALS[ 'TYPO3_DB' ]->UPDATEquery(
						$table
						, 'uid = ' . $uid
						, $data
		);

		switch( $bPromptQueryOnly )
		{
			case(TRUE):
				$this->_DRSprompt( 'Query: ' . $query, $line );
				$this->_DRSprompt( 'Query is for prompt only and will not executed.', $line );
				var_dump( __METHOD__, __LINE__, $query );
				break;
			case(FALSE):
			default:
				$this->_DRSprompt( 'Query: ' . $query, $line );
				$GLOBALS[ 'TYPO3_DB' ]->exec_UPDATEquery(
								$table
								, 'uid = ' . $uid
								, $data
				);
				$this->_aTableSQLLastID[ $table ] = $uid;
				$this->_SQLError( $query, $line );
				break;
		}
	}

	/**
	 * _clearCache( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _clearCache( )
	{
		$query = 'TRUNCATE `cf_cache_pages`';
		$GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$this->_SQLError( $query, __LINE__ );
		
//	Code from below is proper only, if an admin is logged in in the backend.
//		$tce = GeneralUtility::makeInstance( DataHandler::class );
//		$tce->stripslashes_values = false;
//		$tce->admin = 1;
//		$tce->start( [ ], [ ] );
//		$tce->clear_cacheCmd( $cacheCmd );
	}

	/**
	 * _feuserIsLoggedIn( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _feuserIsLoggedIn()
	{
		if( empty( $GLOBALS[ 'TSFE' ]->loginUser ) )
		{
			$prompt = 'fe_user isn\'t logged in.';
			$this->_DRSprompt( $prompt, __LINE__ );
			return false;
		}
		$prompt = 'fe_user is logged in.';
		$this->_DRSprompt( $prompt, __LINE__ );
		return true;
	}

	/**
	 * _getAction( ) :
	 *
	 * @return string
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _getAction()
	{
		// #i0296, dwildt, +
		$sessionData = $this->_sessionGet();
		$action = $sessionData[ 'action' ];
		if( empty( $action ) )
		{
			$default = $this->settings[ 'constants' ][ 'interface' ][ 'action' ][ 'default' ];
			$action = $default;
		}
		return $action;
	}

	/**
	 * _getTCActrlDelete( ) :
	 *
	 * @param string 
	 * @return string
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _getTCActrlDelete( $table )
	{
		$ctrl = $GLOBALS[ 'TCA' ][ $table ][ 'ctrl' ];
		if( isset( $ctrl[ 'delete' ] ) )
		{
			return $ctrl[ 'delete' ];
		}
		return null;
	}

	/**
	 * _getMMlabelUidLocal( ) :
	 *
	 * @param array
	 * @param string
	 * @param string
	 * @return string
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _getMMlabelUidLocal( $config, $table, $field )
	{
		switch( TRUE )
		{
			case(isset( $config[ 'MM_oppositeUsage' ] )):
				$header = 'ERROR: non supported TCA configuration';
				$text = $table . '.' . $field . ' is configured with "MM_oppositeUsage".';
				$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
				die( __METHOD__ . "#" . __LINE__ );
			case(isset( $config[ 'MM_opposite_field' ] )):
				$label = 'uid_foreign';
				break;
			case(!isset( $config[ 'MM_opposite_field' ] )):
				$label = 'uid_local';
			default:
				break;
		}

		return $label;
	}

	/**
	 * _getPid( ) :
	 *
	 * @param string $table
	 * @param integer $uid
	 * @param int $line
	 * @param boolean $bPromptQueryOnly
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _getPid( $table )
	{
		if( !isset( $this->settings[ 'pid' ][ $table ] ) )
		{
			return;
		}
		$typoscriptPlain = $this->settings[ 'pid' ][ $table ];
		$name = $typoscriptPlain[ '_typoScriptNodeValue' ];
		$conf = $this->_oTypoScriptService->convertPlainArrayToTypoScriptArray( $typoscriptPlain );

		$pid = ( int ) $this->_oCObj->cObjGetSingle( $name, $conf );
		if( !empty( $pid ) )
		{
			return $pid;
		}

		$header = 'FATAL ERROR';
		$text = 'pid is empty! Table: ' . $table . '. TypoScript path: settings.pid.' . $table;
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
	}

	/**
	 * _getMMlabelUidForeign( ) :
	 *
	 * @param array
	 * @param string
	 * @param string
	 * @return string
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _getMMlabelUidForeign( $config, $table, $field )
	{
		switch( TRUE )
		{
			case(isset( $config[ 'MM_oppositeUsage' ] )):
				$header = 'ERROR: non supported TCA configuration';
				$text = $table . '.' . $field . ' is configured with "MM_oppositeUsage".';
				$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
				die( __METHOD__ . "#" . __LINE__ );
			case(isset( $config[ 'MM_opposite_field' ] )):
				$label = 'uid_local';
				break;
			case(!isset( $config[ 'MM_opposite_field' ] )):
				$label = 'uid_foreign';
			default:
				break;
		}

		return $label;
	}

	/**
	 * _getTCAconfMMmatchFields( ) :
	 *
	 * @param array $config
	 * @return array data
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _getTCAconfMMmatchFields( $config )
	{
		$data = null;
		foreach( ( array ) $config[ 'MM_match_fields' ] AS $key => $value )
		{
			$data[ $key ] = $value;
		}
		//var_dump(__METHOD__, __LINE__, $data );
		return $data;
	}

	/**
	 * _getUid( ) :
	 *
	 * @return int	$uid
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _getUid( $table = null )
	{
		$table = $this->_getUidTable( $table );

		$sessionData = $this->_sessionGet();
		if( !empty( $sessionData ) )
		{
			return $sessionData[ $table . '.uid' ];
		}

		$header = 'FATAL ERROR: session isn\'t initiated';
		$text = 'This isn\'t proper!';
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
	}

	/**
	 * _getUidTable( ) :
	 *
	 * @param string $table
	 * @return string	$table
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _getUidTable( $table )
	{
		if( !empty( $table ) )
		{
			return $table;
		}

		$table = $this->_sCurrentTable;
		if( !empty( $table ) )
		{
			return $table;
		}

		$header = 'FATAL ERROR: table isn\'t initiated';
		$text = 'This isn\'t proper!';
		$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
	}

	/**
	 * _getWhereData( ):
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _getWhereData( $data )
	{
		$elements = null;
		$where = null;

		foreach( $data as $key => $value )
		{
			$elements[] = $key . ' = "' . $value . '"';
		}

		$where = implode( ' AND ', ( array ) $elements );
		return $where;
	}

	/**
	 * _init( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 7.4.0
	 */
	private function _init()
	{
		$this->_initLL();
		$this->_initExtConf();
		$this->_initObject();
		$this->_setPowermailGP();
		$this->_initTables();
		$this->_initDRS();
		$this->_initTCA();
	}

	/**
	 * _initLL( ) : Init localisation
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _initLL()
	{
		$this->_initObjectPlugin();
		$this->_oPlugin->pi_loadLL( 'EXT:' . $this->extKey . '/' . $this->_sLanguageFilePath );
	}

	/**
	 * _initObject( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initObject()
	{
		$this->_initObjectDRS();
		$this->_initObjectGraphFuncObj();
		$this->_initObjectObjectManager();
		$this->_initObjectCObj();
		$this->_initObjectPowermailParams();
		$this->_initObjectTCA();
		$this->_initObjectTables();
		$this->_initObjectTypoScriptService();
	}

	/**
	 * _initObjectCObj( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initObjectCObj()
	{
		$this->_oCObj = $this->_oObjectManager->get( 'TYPO3\\CMS\\Frontend\\ContentObject\\ContentObjectRenderer' );
	}

	/**
	 * _initObjectDRS( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initObjectDRS()
	{
		$this->_oDRS = GeneralUtility::makeInstance( 'Netzmacher\\Browser\\Utility\\FrontendEditing\\DRS' );
	}

	/**
	 * _initObjectGraphFuncObj( ) :
	 *
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.0
	 */
	private function _initObjectGraphFuncObj()
	{
		$this->_oGraphFuncObj = GeneralUtility::makeInstance( \TYPO3\CMS\Core\Imaging\GraphicalFunctions::class );
		$this->_oGraphFuncObj->init();
	}

	/**
	 * _initObjectObjectManager( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initObjectObjectManager()
	{
		$this->_oObjectManager = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Extbase\\Object\\ObjectManager' );
	}

	/**
	 * _initObjectPlugin( ) : Init localisation
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _initObjectPlugin()
	{
		$this->_oPlugin = GeneralUtility::makeInstance( \TYPO3\CMS\Frontend\Plugin\AbstractPlugin::class );
	}

	/**
	 * _initObjectPowermailParams( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initObjectPowermailParams()
	{
		$this->_oParamsPowermail = GeneralUtility::makeInstance( 'Netzmacher\\Browser\\Utility\\FrontendEditing\\Params\\Powermail' );
	}

	/**
	 * _initObjectTables( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initObjectTables()
	{
		$this->_oTables = GeneralUtility::makeInstance( 'Netzmacher\\Browser\\Utility\\FrontendEditing\\Tables' );
	}

	/**
	 * _initObjectTCA( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initObjectTCA()
	{
		$this->_oTCA = GeneralUtility::makeInstance( 'Netzmacher\\Browser\\Utility\\FrontendEditing\\TCA' );
	}

	/**
	 * _initObjectTypoScriptService( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _initObjectTypoScriptService()
	{
		$this->_oTypoScriptService = GeneralUtility::makeInstance( 'TYPO3\\CMS\\Extbase\\Service\\TypoScriptService' );
	}

	/**
	 * _initDRS( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initDRS()
	{
		$this->_oTCA->setDRS( $this->_oDRS );
		$this->_oTables->setDRS( $this->_oDRS );
	}

	/**
	 * _initExtConf( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initExtConf()
	{
		$this->_aExtConf = unserialize( $GLOBALS[ 'TYPO3_CONF_VARS' ][ 'EXT' ][ 'extConf' ][ 'browser' ] );
	}

	/**
	 * _initTables( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initTables()
	{
		$this->_aTables = array();

		// if fe_users exits, it should be the firt element in $_aTables
		if( isset( $this->settings[ 'mapping' ][ 'fe_users' ] ) )
		{
			$this->_aTables[ 'fe_users' ] = array(
					'uid' => 'new',
					'query' => array(
							'fields' => array(
							),
					),
					'postProcess' => array(
					),
			);
		}

		foreach( array_keys( ( array ) $this->settings[ 'mapping' ] ) AS $table )
		{
			switch( TRUE )
			{
				case( $table == 'fe_users'):
				case( $table == '_typoScriptNodeValue'):
					continue;
				default:
					$this->_aTables[ $table ] = array(
							'uid' => 'new',
							'query' => array(
									'fields' => array(
									),
							),
							'postProcess' => array(
							),
					);
					break;
			}
		}

//    var_dump( __METHOD__, __LINE__, array_keys( $this->settings[ 'mapping' ] ), $this->_aTables );
	}

	/**
	 * _initTCA( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _initTCA()
	{
		$this->_oTables->setTCA( $this->_oTCA );
	}

	/**
	 * _powermailActionIsConfirmation( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _powermailActionIsConfirmation()
	{
		switch( true )
		{
			case(!isset( $this->_aPowermailGP[ 'action' ] )):
				return false;
			case($this->_aPowermailGP[ 'action' ] != 'confirmation'):
				return false;
			default :
				return true;
		}
	}

	/**
	 * _powermailActionIsCreate( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _powermailActionIsCreate()
	{
		switch( true )
		{
			case(!isset( $this->_aPowermailGP[ 'action' ] )):
				return false;
			case($this->_aPowermailGP[ 'action' ] != 'create'):
				return false;
			default :
				return true;
		}
	}

	/**
	 * _powermailActionIsForm( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _powermailActionIsForm()
	{
		switch( true )
		{
			case(!isset( $this->_aPowermailGP[ 'action' ] )):
				return false;
			case($this->_aPowermailGP[ 'action' ] != 'form'):
				return false;
			default :
				return true;
		}
	}

	/**
	 * _powermailHideFormByCss( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _powermailHideFormByCss()
	{
		$this->_initObjectObjectManager();
		$this->_initObjectCObj();

		$css = null;
		$configuration = [
				'table' => 'tt_content',
				'pidInList' => $GLOBALS[ 'TSFE' ]->id,
				'select.' => [
						'max' => 1,
						'where' => 'list_type = "powermail_pi1"',
				],
				'renderObj' => 'TEXT',
				'renderObj.' => [
						'field' => 'uid',
				]
		];
		$uid = $this->_oCObj->cObjGetSingle( 'CONTENT', $configuration );

		if( !empty( $uid ) )
		{
			$css = '
      <style type="text/css">
        #c' . $uid . ' {
          display: none;
        }
      </style>
      ';
		}
		return $css;
	}

	/**
	 * _powermailPrefill( ) :
	 *
	 * @return void
	 * @access private
	 * @internal #i0293
	 * @version 8.0.21
	 * @since 8.0.17
	 */
	private function _powermailPrefill()
	{
		switch( true )
		{
			case($this->_getAction() == 'add'):
				$this->_powermailPrefillInitObject();
//		var_dump(__METHOD__, __LINE__, $this->_aPowermailGP);
				$this->_oPrefill->main( $this->settings );
				break;
			case($this->_getAction() == 'update'):
				if( !$this->_feuserIsLoggedIn() )
				{
					return;
				}
				$this->_powermailPrefillInitObject();
				$this->_oPrefill->main( $this->settings );
				break;
			default:
				// Do nothing
				break;
		}
		return;
	}

	/**
	 * _powermailPrefillInitObject( ) :
	 *
	 * @return void
	 * @access private
	 * @internal #i0293
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _powermailPrefillInitObject()
	{
		$this->_oPrefill = GeneralUtility::makeInstance( 'Netzmacher\\Browser\\Utility\\FrontendEditing\\Prefill' );
		$this->_oPrefill->setAction( $this->_getAction() );
		$this->_powermailPrefillInitObjectDRS();
		$this->_powermailPrefillInitObjectTCA();
	}

	/**
	 * _powermailPrefillInitObjectDRS( ) :
	 *
	 * @return void
	 * @access private
	 * @internal #i0293
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _powermailPrefillInitObjectDRS()
	{
		$this->_oPrefill->setDRS( $this->_oDRS );
	}

	/**
	 * _powermailPrefillInitObjectTCA( ) :
	 *
	 * @return void
	 * @access private
	 * @internal #i0293
	 * @version 8.0.17
	 * @since 8.0.17
	 */
	private function _powermailPrefillInitObjectTCA()
	{
		$this->_oPrefill->setTCA( $this->_oTCA );
	}

	/**
	 * _powermailUnsetParamAction( ) :
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _powermailUnsetParamAction()
	{
		unset( $_GET[ 'tx_powermail_pi1' ][ 'action' ] );
		unset( $_POST[ 'tx_powermail_pi1' ][ 'action' ] );
	}

	/**
	 * _processForeignTable( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.8
	 * @since 7.4.0
	 */
	private function _processForeignTable( $table, $field )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		switch( TRUE )
		{
			case(isset( $config[ 'MM' ] )):
				$this->_processForeignTableMM( $table, $field );
				break;
			case($config[ 'foreign_table' ] == 'sys_file_reference'):
				// #i0271, 161120, dwildt, 2+
				$this->_processFAL( $table, $field );
				break;
			default:
				$this->_processForeignTableCSV( $table, $field );
				break;
		}
	}

	/**
	 * _processForeignTableCSV( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processForeignTableCSV( $table, $field )
	{
		$record = $this->_oTables->getTableRecordKeyValues( $table );

		// Get update date
		$value = $record[ $field ];
		if( is_array( $value ) )
		{
			$value = implode( ', ', $value );
		}
		$data = array(
				$field => $value
		);

		// Get the uid
		$tableField = $table . '.uid';
		$uid = $this->_oCObj->data[ $tableField ];
		$this->_SQLUpdate( $table, $uid, $data, __LINE__ );
	}

	/**
	 * _processForeignTableMM( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processForeignTableMM( $table, $field )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );

		$labelUidLocal = $this->_getMMlabelUidLocal( $config, $table, $field );
		$labelUidForeign = $this->_getMMlabelUidForeign( $config, $table, $field );

		$this->_processForeignTableMMDelete( $table, $field, $labelUidLocal );
		$this->_processForeignTableMMInsert( $table, $field, $labelUidLocal, $labelUidForeign );
	}

	/**
	 * _processForeignTableMMDelete( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.18
	 */
	private function _processForeignTableMMDelete( $table, $field, $labelUidLocal )
	{
		$data = null;
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		$mmTable = $config[ 'MM' ];

		$data = $this->_getTCAconfMMmatchFields( $config );

		// Set uid
		$tableField = $table . '.uid';
		$uidLocal = $this->_oCObj->data[ $tableField ];
		$data[ $labelUidLocal ] = $uidLocal;

		$where = $this->_getWhereData( $data );

		$this->_SQLDelete( $mmTable, $where, __LINE__, 0 );
	}

	/**
	 * _processForeignTableMMInsert( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _processForeignTableMMInsert( $table, $field, $labelUidLocal, $labelUidForeign )
	{
		$action = $this->_getAction();
		switch( $action )
		{
			case('add'):
			case('update'):
				// Follow the workflow
				break;
			case('deleterecord'):
				// Do nothing
				return;
			default:
				$this->_DRSswitchActionDie( $action, __METHOD__, __LINE__ );
				return;
		}

//var_dump(__METHOD__, __LINE__, $staticInsertData, $record, $aValues);
//die();
		$data = null;
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		$mmTable = $config[ 'MM' ];

		$data = $this->_getTCAconfMMmatchFields( $config );

		// Set uid
		$tableField = $table . '.uid';
		$uidLocal = $this->_oCObj->data[ $tableField ];
		$data[ $labelUidLocal ] = $uidLocal;

		// Set values
		$record = $this->_oTables->getTableRecordKeyValues( $table );
		$aValues = $record[ $field ];
		if( !is_array( $aValues ) )
		{
			$aValues = array(
					0 => $aValues
			);
		}

		// Inserts
		foreach( ( array ) $aValues AS $uidForeign )
		{
			$data[ $labelUidForeign ] = $uidForeign;
			//var_dump( __METHOD__, __LINE__, $mmTable, $data );
			$this->_SQLInsert( $mmTable, $data, __LINE__, 0 );
		}
	}

	/**
	 * _processInternalType( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processInternalType( $table, $field )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		switch( $config[ 'internal_type' ] )
		{
			case('db'):
				$this->_processInternalTypeDB( $table, $field );
				break;
			case('file'):
				$this->_processInternalTypeFile( $table, $field );
				break;
			default:
				$tableField = $table . '.' . $field;
				$header = 'ERROR: undefined key';
				$key = $config[ 'internal_type' ];
				$text = $tableField . ': Key for internal_type isn\'t defined! Key is "' . $key . '"';
				$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
				break;
		}
	}

	/**
	 * _processInternalTypeDB( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processInternalTypeDB( $table, $field )
	{
		$this->_processForeignTable( $table, $field );
	}

	/**
	 * _processFAL( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.0
	 */
	private function _processFAL( $table, $field )
	{
		if( !$this->_processInternalTypeFileCopy( $table, $field, $this->_sUploadfolder ) )
		{
			return;
		}

		$this->_processFALSysFile( $table, $field );
		$this->_processFALSysFileMetadata();
		$this->_processFALSysFileReference( $table, $field );

		$this->_processFALMainUpdate( $table, $field );
	}

	/**
	 * _processFALMainUpdate( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.0
	 */
	private function _processFALMainUpdate( $table, $field )
	{
		$data = array(
				$field => 1,
		);
		$uid = $this->_sCurrentTableRecordUid[ $table ];
		$this->_SQLUpdate( $table, $uid, $data, __LINE__ );
	}

	/**
	 * _processFALSysFile( )
	 *
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.0
	 */
	private function _processFALSysFile( $table, $field )
	{
		$insertTable = 'sys_file';

		$tableField = $table . '.' . $field;
		$name = $this->_oCObj->data[ $tableField ];
		$identifier = $this->_sCurrentFilePath;
		list($pathWoExt, $extension) = explode( '.', $this->_sCurrentFilePath );
		unset( $pathWoExt );

		$data = array(
				'creation_date' => time(),
				'extension' => $extension,
				'identifier' => $identifier,
				'modification_date' => time(),
				'name' => $name,
				'tstamp' => time(),
		);
		$this->_SQLInsert( $insertTable, $data, __LINE__ );
	}

	/**
	 * _processFALSysFileMetadata( )
	 *
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.0
	 */
	private function _processFALSysFileMetadata()
	{
		$insertTable = 'sys_file_metadata';

		$info = $this->_oGraphFuncObj->getImageDimensions( $this->_sCurrentFileAbsPath );
		$file = $this->_aTableSQLLastID[ 'sys_file' ];

		$data = array(
				'crdate' => time(),
				'cruser_id' => 0,
				'file' => $file,
				'height' => $info[ 1 ],
				'title' => '',
				'tstamp' => time(),
				'width' => $info[ 0 ],
		);
		$this->_SQLInsert( $insertTable, $data, __LINE__ );
	}

	/**
	 * _processFALSysFileReference( )
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.0
	 */
	private function _processFALSysFileReference( $table, $field )
	{
		$insertTable = 'sys_file_reference';

		$fieldname = $field;
		$tablenames = $table;
		$cruser_id = 0;
		$pid = $this->_getPid( $table );
		$uid_foreign = $file = $this->_aTableSQLLastID[ $table ];
		$uid_local = $file = $this->_aTableSQLLastID[ 'sys_file' ];

		$data = array(
				'crdate' => time(),
				'cruser_id' => $cruser_id,
				'fieldname' => $fieldname,
				'pid' => $pid,
				'tablenames' => $tablenames,
				'table_local' => 'sys_file',
				'tstamp' => time(),
				'uid_foreign' => $uid_foreign,
				'uid_local' => $uid_local,
		);
		$this->_SQLInsert( $insertTable, $data, __LINE__ );
	}

	/**
	 * _processInternalTypeFile( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processInternalTypeFile( $table, $field )
	{
		if( !$this->_processInternalTypeFileCopy( $table, $field ) )
		{
			return;
		}

		$this->_processInternalTypeFileUpdate( $table, $field );
	}

	/**
	 * _processInternalTypeFileUpdate( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processInternalTypeFileUpdate( $table, $field )
	{
		$tableField = $table . '.' . $field;
		$data = array(
				$field => $this->_oCObj->data[ $tableField ]
		);
		$uid = $this->_sCurrentTableRecordUid[ $table ];
		$this->_SQLUpdate( $table, $uid, $data, __LINE__ );

//    var_dump( __METHOD__, __LINE__, $table, $field, $file, $this->_sCurrentTableRecordUid );
//    $header = 'ERROR: undefined key';
//    $key = $config[ 'internal_type' ];
//    $text = $tableField . ': Key for internal_type isn\'t defined! Key is "' . $key . '"';
//    $this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
	}

	/**
	 * _processInternalTypeFileCopy( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $uploadfolder
	 * @return boolean
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 7.4.0
	 */
	private function _processInternalTypeFileCopy( $table, $field, $uploadfolder = NULL )
	{
		$srce = $this->_processInternalTypeFileCopySrce( $table, $field );
		$dest = $this->_processInternalTypeFileCopyDest( $table, $field, $uploadfolder );
		try
		{
			copy( $srce, $dest );
		}
		catch( \Exception $e )
		{
			$prompt = $e->getMessage();
			$bSuccess = FALSE;
			$this->_processInternalTypeFileCopyDRS( $srce, $dest, $bSuccess, $prompt );
			return FALSE;
		}

		$bSuccess = TRUE;
		$this->_processInternalTypeFileCopyDRS( $srce, $dest, $bSuccess, $prompt );
		return TRUE;
	}

	/**
	 * _processInternalTypeFileCopyDRS( ) :
	 *
	 * @param string $srce
	 * @param string $dest
	 * @param boolean $bSuccess
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processInternalTypeFileCopyDRS( $srce, $dest, $bSuccess, $prompt )
	{
		switch( $bSuccess )
		{
			case( TRUE ):
				$prompt = 'File is moved from ' . $srce . ' to ' . $dest;
				$this->_DRSprompt( $prompt, __LINE__, -1 );
				break;
			case( FALSE ):
			default:
				$prompt = 'ERROR! File couldn\'t moved from "' . $srce . '" to "' . $dest . '". '
								. $prompt
				;
				$this->_DRSprompt( $prompt, __LINE__, 3 );
				break;
		}
	}

	/**
	 * _processInternalTypeFileCopyDest( ) : Rteurns the absolute path for the file destination
	 *
	 * @param string $table
	 * @param string $field
	 * @param string $uploadfolder
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 7.4.0
	 */
	private function _processInternalTypeFileCopyDest( $table, $field, $uploadfolder )
	{
		$tableField = $table . '.' . $field;
		$file = $this->_oCObj->data[ $tableField ];

		$path = $this->_processInternalTypeFileCopyDestUploadfolder( $table, $field, $uploadfolder ) . '/' . $file;
		$absPath = GeneralUtility::getFileAbsFileName( $path );
//    var_dump( __METHOD__, __LINE__, $tableField, $file, $path, $absPath );
		$this->_sCurrentFilePath = $path;
		$this->_sCurrentFileAbsPath = $absPath;
		return $absPath;
	}

	/**
	 * _processInternalTypeFileCopyDest( ) : Rteurns the absolute path for the file destination
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.8
	 */
	private function _processInternalTypeFileCopyDestUploadfolder( $table, $field, $uploadfolder = NULL )
	{
		switch( TRUE )
		{
			case($uploadfolder === NULL):
				$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
				$uploadfolder = $config[ 'uploadfolder' ];
				break;
			case($uploadfolder !== NULL):
				// take the given uploadfolder
				break;
		}

		return $uploadfolder;
	}

	/**
	 * _processInternalTypeFileCopySrce( ) : Returns the absolute path to the file uploaded by powermail
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _processInternalTypeFileCopySrce( $table, $field )
	{
		$tableField = $table . '.' . $field;
		$file = $this->_oCObj->data[ $tableField ];
		$path = $this->_sPowermailPath . '/' . $file;
		$absPath = GeneralUtility::getFileAbsFileName( $path );
//var_dump( __METHOD__, __LINE__, $tableField, $this->_oCObj->data, $path, $absPath );
		return $absPath;
	}

	/**
	 * _renderData( ) :
	 *
	 * @param	array
	 * @return array
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderData( $data )
	{
		foreach( ( array ) $data as $field => $value )
		{
			$data[ $field ] = $this->_renderDataValue( $field, $value );
		}
		//var_dump( __METHOD__, __LINE__, $data );
		//die();
		return $data;
	}

	/**
	 * _renderDataValue( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderDataValue( $field, $value )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $this->_sCurrentTable, $field );
		switch( $config[ 'type' ] )
		{
			case( 'input'):
				$value = $this->_renderDataValueInput( $field, $value );
				break;
			default:
				// Do nothing
				break;
		}

		return $value;
	}

	/**
	 * _renderDataValueInput( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderDataValueInput( $field, $value )
	{
		$config = BackendUtility::getTcaFieldConfiguration( $this->_sCurrentTable, $field );
		switch( true )
		{
			case( strpos( $config[ 'eval' ], 'datetime' ) !== false ):
				$value = $this->_renderDataValueEvalDatetime( $value );
				break;
			default:
				// Do nothing
				break;
		}
		return $value;
	}

	/**
	 * _renderDataValueEvalDatetime( ) : 
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _renderDataValueEvalDatetime( $value )
	{
		//$config = BackendUtility::getTcaFieldConfiguration( $this->_sCurrentTable, $field );
		$value = strtotime( $value );
		return $value;
	}

	/**
	 * _requirements( ) : 
	 *
	 * @param string $action
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _requirements( $action )
	{
//				var_dump(__METHOD__, __LINE__, $action );
		if( !$this->_requirementsFeUser( $action ) )
		{
			return false;
		}
		if( !$this->_requirementsUid( $action ) )
		{
			return false;
		}
		return true;
	}

	/**
	 * _requirementsFeUser( ) : 
	 *
	 * @param string $action
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _requirementsFeUser( $action )
	{
//				var_dump(__METHOD__, __LINE__, $action );

		switch( $action )
		{
			case('add'):
			case('copy'):
			case('delete'):
			case('index'):
			case('update'):
				$accessWoFeUser = $this->settings[ 'constants' ][ 'interface' ][ 'access' ][ $action . 'WoFeUser' ];
				if( $accessWoFeUser )
				{
					return true;
				}
				if( $this->_feuserIsLoggedIn() )
				{
					return true;
				}
				$this->_requirementsViewAssign( $action, 'FeUser' );
				return false;
			default:
				$prompt = '
      <h1 style="color:red;">
        Action isn\'t defined
      </h1>
      <p>
        Action: "' . $action . '"
      </p>
      <p>
        Sorry for the trouble. Browser - TYPO3 without PHP<br />
        Error occures here: ' . __METHOD__ . ' at #' . __LINE__ . '
      </p>
      ';
				die( $prompt );
		}
	}

	/**
	 * _requirementsUid( ) : 
	 *
	 * @param string $action
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _requirementsUid( $action )
	{
//				var_dump(__METHOD__, __LINE__, $action );

		switch( $action )
		{
			case('add'):
				if( $this->_requirementsUidWo() )
				{
					return true;
				}
				$this->_requirementsViewAssign( $action, 'Uid' );
				return false;
			case('copy'):
			case('delete'):
			case('update'):
				if( $this->_requirementsUidWi() )
				{
					return true;
				}
				$this->_requirementsViewAssign( $action, 'Uid' );
				return false;
			case('index'):
				// do nothing
				return true;
			default:
				$prompt = '
      <h1 style="color:red;">
        Action isn\'t defined
      </h1>
      <p>
        Action: "' . $action . '"
      </p>
      <p>
        Sorry for the trouble. Browser - TYPO3 without PHP<br />
        Error occures here: ' . __METHOD__ . ' at #' . __LINE__ . '
      </p>
      ';
				die( $prompt );
		}
	}

	/**
	 * _requirementsUidWi( ) : 
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _requirementsUidWi()
	{
		$showUid = $this->settings[ 'constants' ][ 'interface' ][ 'param' ][ 'showUid' ];
		$arguments = $this->request->getArguments();
		$uid = ( int ) $arguments[ $showUid ];
//var_dump( __METHOD__, __LINE__, $this->settings, $showUid, $arguments, $uid);
		if( !empty( $uid ) )
		{
			return true;
		}
		return false;
	}

	/**
	 * _requirementsUidWo( ) : 
	 *
	 * @return boolean
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _requirementsUidWo()
	{
		$showUid = $this->settings[ 'constants' ][ 'interface' ][ 'param' ][ 'showUid' ];
		$arguments = $this->request->getArguments();
		if( !isset( $arguments[ $showUid ] ) )
		{
			return true;
		}
		return false;
	}

	/**
	 * _requirementsViewAssign( ) : 
	 *
	 * @param string $action
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _requirementsViewAssign( $action, $requirement )
	{
//		var_dump(__METHOD__, __LINE__, $action );
		$this->view->assignMultiple(
						array(
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelCancel' ),
								'buttonCancelClass' => 'btn btn-primary button expand',
								'class' => 'alert',
								'css' => $this->_powermailHideFormByCss(),
								'header' => $this->_oPlugin->pi_getLL( 'requirements' . ucfirst( $action ) . 'Header' ),
								'pid' => $this->settings[ 'constants' ][ 'interface' ][ 'page' ][ 'list' ],
								'prompt' => $this->_oPlugin->pi_getLL( 'requirements' . ucfirst( $action ) . 'Prompt' . $requirement ),
								'promptAlertBox' => 'true',
								'promptButtons' => 'true',
								'promptButtonDelete' => 'false',
						)
		);
	}

	/**
	 * _sessionDataSpace( ): Get the name of the session data space.
	 *                        The name is user, if a frontend user is logged in.
	 *                        The name is ses, if any frontend user isn't logged in.
	 *
	 * @return	string		user || ses
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _sessionDataSpace()
	{
		switch( $GLOBALS[ 'TSFE' ]->loginUser )
		{
			case( true ):
				return 'user';
			default:
				return 'ses';
		}
	}

	/**
	 * _sessionGet( ): 
	 *
	 * @return	string		user || ses
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _sessionGet()
	{
		$sessionDataSpace = $this->_sessionDataSpace();
		$sessionData = $GLOBALS[ 'TSFE' ]->fe_user->getKey( $sessionDataSpace, 'tx_browser_pi6' );
		return $sessionData;
	}

	/**
	 * _sessionSet( ): 
	 *
	 * @return	string		user || ses
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _sessionSet( $action = null )
	{
		$sessionDataSpace = $this->_sessionDataSpace();
		$sessionData = [
				'action' => $action,
		];

		$uid = $this->_sessionSetTableUid();
		if( !empty( $uid ) )
		{
			$table = $this->settings[ 'constants' ][ 'interface' ][ 'param' ][ 'table' ];
			$sessionData[ $table . '.uid' ] = $uid;
		}

//		var_dump( __METHOD__, __LINE__, $sessionData );
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( $sessionDataSpace, 'tx_browser_pi6', $sessionData );
		$GLOBALS[ "TSFE" ]->fe_user->sesData_change = true;
		$GLOBALS[ "TSFE" ]->fe_user->storeSessionData();
	}

	/**
	 * _sessionSetTableUid( ): 
	 *
	 * @return	string		user || ses
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _sessionSetTableUid()
	{
		$table = $this->settings[ 'constants' ][ 'interface' ][ 'param' ][ 'table' ];
		$sessionData = $this->_sessionGet();
		$uid = $sessionData[ $table . '.uid' ];
		if( $uid > 0 )
		{
			return $uid;
		}

		//var_dump( __METHOD__, __LINE__, $showUid );
		$showUid = $this->settings[ 'constants' ][ 'interface' ][ 'param' ][ 'showUid' ];
		$arguments = $this->request->getArguments();
		$uid = ( int ) $arguments[ $showUid ];
		if( !empty( $uid ) )
		{
			return $uid;
		}

		return null;
	}

	/**
	 * _sessionUnset( ): 
	 *
	 * @return	string		user || ses
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _sessionUnset()
	{
		$sessionDataSpace = $this->_sessionDataSpace();
		$sessionData = [
		];
		$GLOBALS[ 'TSFE' ]->fe_user->setKey( $sessionDataSpace, 'tx_browser_pi6', $sessionData );
		$GLOBALS[ "TSFE" ]->fe_user->storeSessionData();
	}

	/**
	 * _setCObjStart( ) :
	 *
	 * @param string $table
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _setCObjStart( $table )
	{
		$data = $this->_setCObjStartData();
		$this->_DRSpromptData( $data, __LINE__ );
		$this->_oCObj->start( $data, $table );
	}

	/**
	 * _setCObjStartData( ) :
	 *
	 * @return array
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _setCObjStartData()
	{
		$data = $this->_setCObjStartDataPowermail();
		return $data;
	}

	/**
	 * _setCObjStartDataPowermail( ) :
	 *
	 * @return array
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _setCObjStartDataPowermail()
	{
		foreach( ( array ) $this->_aPowermailGP AS $key => $value )
		{
			if( is_array( $value ) )
			{
				$value = implode( ', ', $value );
			}
			$data[ $key ] = $value;
			list($table, $field) = explode( '__', $key );
			if( $field )
			{
				$tableField = $table . '.' . $field;
				$data[ $tableField ] = $value;
			}
		}
		return $data;
	}

	/**
	 * _setCurrentTableRecordUid( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _setCurrentTableRecordUid()
	{
		$table = $this->_sCurrentTable;
		$uid = $this->_aTableSQLLastID[ $table ];
		$tableField = $table . '.uid';

		$this->_sCurrentTableRecordUid[ $table ] = $uid;
		$this->_aTables[ $this->_sCurrentTable ][ 'uid' ] = $uid;
		$this->_oCObj->data[ $tableField ] = $uid;
		$this->_DRSpromptData( array( $tableField => $uid ), __LINE__ );
	}

	/**
	 * _setPowermailGP( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _setPowermailGP()
	{
		$this->_aPowermailGP = $this->_oParamsPowermail->getGP();
	}

	/**
	 * _tables( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.17
	 * @since 7.4.0
	 */
	private function _tables()
	{
		$action = $this->_getAction();
		switch( $action )
		{
			case('add'):
			case('copy'):
			case('deleterecord'):
			case('update'):
				// Follow the workflow
				break;
			default:
				$this->_DRSswitchActionDie( $action, __METHOD__, __LINE__ );
				break;
		}

		$this->_setCObjStart( 'fe_users' );
		$this->_oTables->setCObj( $this->_oCObj );
		foreach( array_keys( ( array ) $this->_aTables ) AS $this->_sCurrentTable )
		{
			$this->_oTables->main( $this->_sCurrentTable, $this->settings, $this );
			if( $action == 'copy' )
			{
				$this->_table();
				continue;
			}
			if( $action == 'deleterecord' )
			{
				$this->_table();
				continue;
			}
			//var_dump( __METHOD__, __LINE__, $this->_sCurrentTable, $this->_oTables->getKeyValues() );
			if( !empty( $this->_oTables->getKeyValues() ) )
			{
				$this->_table();
				continue;
			}
			// 170127, dwildt, 4+
			if( $this->_sCurrentTable == 'fe_users' )
			{
				continue;
			}

			// * #i0230, 160217, dwildt
			$header = 'FATAL ERROR';
			$prompt = 'No data transferred! Table: ' . $this->_sCurrentTable . '<br />'
							. 'Error can occur'
							. '<ul>'
							. '<li>if plugin.tx_browser_pi6.settings.mapping.' . $this->_sCurrentTable . ' is missing</li>'
							. '<li>if plugin.tx_browser_pi6.settings.mapping.' . $this->_sCurrentTable . ' is configured, but any ' . $this->_sCurrentTable . ' field isn\'t used in the powermail form</li>'
							. '<li>if plugin.tx_browser_pi6.settings.mapping.' . $this->_sCurrentTable . '.allowedFields is empty</li>'
							. '</ul>'
			;
			$this->_zzDieWiPrompt( $header, $prompt, __METHOD__, __LINE__ );
		}
		$this->_tablesPostProcess();
	}

	/**
	 * _tablesForeignCopy( ) :
	 *
	 * @param integer $uidSrce  source uid
	 * @param integer $uidDest	destination uid
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignCopy( $uidSrce, $uidDest )
	{
		$table = $this->_sCurrentTable;

		foreach( array_keys( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] ) as $foreignTable )
		{
			$config = BackendUtility::getTcaFieldConfiguration( $table, $foreignTable );
			if( !isset( $config[ 'foreign_table' ] ) )
			{
				continue;
			}
			$this->_tablesForeignCopyTable( $config, $foreignTable, $uidSrce, $uidDest );
		}
	}

	/**
	 * _tablesForeignCopyTable( ) :
	 *
	 * @param array $config
	 * @param string $foreignTable
	 * @param integer $uidSrce  source uid
	 * @param integer $uidDest	destination uid
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignCopyTable( $config, $foreignTable, $uidSrce, $uidDest )
	{
		switch( TRUE )
		{
			case(isset( $config[ 'MM' ] )):
				//var_dump( __METHOD__, __LINE__, $foreignTable, 'MM' );
				//$this->_tablesForeignCopyTableMM( $table, $foreignTable );
				$this->_tablesForeignCopyTableMM( $foreignTable, $uidSrce, $uidDest );
				break;
			case($config[ 'foreign_table' ] == 'sys_file_reference'):
				//var_dump( __METHOD__, __LINE__, $foreignTable, 'sys_file_reference' );
				$this->_tablesForeignCopyTableFAL( $foreignTable, $uidSrce, $uidDest );
				//$this->_processFAL( $table, $field );
				break;
			case($foreignTable == 'l10n_parent'):
			case($foreignTable == 'sys_language_uid'):
				//var_dump( __METHOD__, __LINE__, $foreignTable );
				$prompt = 'Table won\'t handled: ' . $foreignTable . '. This is OK.';
				$this->_DRSprompt( $prompt, __LINE__, 0 );
				break;

			default:
				$prompt = 'Table won\'t handled: ' . $foreignTable . '. Requirements doesn\'t match. Maybe this behaviour isn\'t proper.';
				$this->_DRSprompt( $prompt, __LINE__, 0 );
				break;
		}
	}

	/**
	 * _tablesForeignCopyTableMM( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @param integer $uidSrce  source uid
	 * @param integer $uidDest	destination uid
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignCopyTableMM( $field, $uidSrce, $uidDest )
	{
		$table = $this->_sCurrentTable;
		$data = null;
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		$mmTable = $config[ 'MM' ];

		$data = $this->_getTCAconfMMmatchFields( $config );

		$labelUidLocal = $this->_getMMlabelUidLocal( $config, $table, $field );
		$set = $labelUidLocal . ' = ' . $uidDest;

		$data[ $labelUidLocal ] = $uidSrce;
		$where = $this->_getWhereData( $data );

		$this->_SQLCopyMM( $mmTable, $set, $where, $uidSrce, $uidDest, __LINE__, 0 );
	}

	/**
	 * _tablesForeignCopyTableFAL( ) :
	 *
	 * @param string $field
	 * @param integer $uidSrce  source uid
	 * @param integer $uidDest	destination uid
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignCopyTableFAL( $field, $uidSrce, $uidDest )
	{
		$table = $this->_sCurrentTable;
		$data = null;
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		$mmTable = 'sys_file_reference';

		if( $config[ 'foreign_table' ] != 'sys_file_reference' )
		{
			$header = 'FATAL ERROR';
			$text = 'foreign_table != sys_file_reference';
			$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
		}

		$set = 'uid = null, uid_foreign = ' . $uidDest;

		$data = array(
				'uid_foreign' => $uidSrce,
				'tablenames' => $table,
				'fieldname' => $field,
		);
		$where = $this->_getWhereData( $data );

		$this->_SQLCopyMM( $mmTable, $set, $where, $uidSrce, $uidDest, __LINE__, 0 );
	}

	/**
	 * _tablesForeignDelete( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignDelete()
	{
		$table = $this->_sCurrentTable;

		foreach( array_keys( ( array ) $GLOBALS[ 'TCA' ][ $table ][ 'columns' ] ) as $foreignTable )
		{
			$config = BackendUtility::getTcaFieldConfiguration( $table, $foreignTable );
			if( !isset( $config[ 'foreign_table' ] ) )
			{
				continue;
			}
			$this->_tablesForeignDeleteTable( $config, $foreignTable );
		}
	}

	/**
	 * _tablesForeignDeleteTable( ) :
	 *
	 * @param array $config
	 * @param string $foreignTable
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignDeleteTable( $config, $foreignTable )
	{
		switch( TRUE )
		{
			case(isset( $config[ 'MM' ] )):
				//var_dump( __METHOD__, __LINE__, $foreignTable, 'MM' );
				//$this->_tablesForeignDeleteTableMM( $table, $foreignTable );
				$this->_tablesForeignDeleteTableMM( $foreignTable );
				break;
			case($config[ 'foreign_table' ] == 'sys_file_reference'):
				//var_dump( __METHOD__, __LINE__, $foreignTable, 'sys_file_reference' );
				$this->_tablesForeignDeleteTableFAL( $foreignTable );
				//$this->_processFAL( $table, $field );
				break;
			case($foreignTable == 'l10n_parent'):
			case($foreignTable == 'sys_language_uid'):
				//var_dump( __METHOD__, __LINE__, $foreignTable );
				$prompt = 'Table won\'t handled: ' . $foreignTable . '. This is OK.';
				$this->_DRSprompt( $prompt, __LINE__, 0 );
				break;

			default:
				$prompt = 'Table won\'t handled: ' . $foreignTable . '. Requirements doesn\'t match. Maybe this behaviour isn\'t proper.';
				$this->_DRSprompt( $prompt, __LINE__, 0 );
				break;
		}
	}

	/**
	 * _tablesForeignDeleteTableMM( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignDeleteTableMM( $field )
	{
		$table = $this->_sCurrentTable;
		$data = null;
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		$mmTable = $config[ 'MM' ];

		$data = $this->_getTCAconfMMmatchFields( $config );

		$labelUidLocal = $this->_getMMlabelUidLocal( $config, $table, $field );

		// Set uid
		$uidLocal = $this->_getUid( $table );
		$data[ $labelUidLocal ] = $uidLocal;

		// params for where statement
		$where = $this->_getWhereData( $data );

		$this->_SQLDelete( $mmTable, $where, __LINE__, 0 );
	}

	/**
	 * _tablesForeignDeleteTableFAL( ) :
	 *
	 * @param string $table
	 * @param string $field
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tablesForeignDeleteTableFAL( $field )
	{
		$table = $this->_sCurrentTable;
		$data = null;
		$config = BackendUtility::getTcaFieldConfiguration( $table, $field );
		if( $config[ 'foreign_table' ] != 'sys_file_reference' )
		{
			$header = 'FATAL ERROR';
			$text = 'foreign_table != sys_file_reference';
			$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
		}

		$data = array(
				'uid_foreign' => $this->_getUid( $table ),
				'tablenames' => $table,
				'fieldname' => $field,
		);

		$where = $this->_getWhereData( $data );

		$mmTable = 'sys_file_reference';
		$this->_SQLDelete( $mmTable, $where, __LINE__, 0 );
	}

	/**
	 * _tablesPostProcess( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _tablesPostProcess()
	{
		$aTCATableField = $this->_oTCA->getTableField();
		foreach( array_keys( ( array ) $aTCATableField ) AS $TCATable )
		{
			if( !in_array( $TCATable, array_keys( ( array ) $this->_aTables ) ) )
			{
				$prompt = 'Nothing to do: ' . $TCATable . ' isn\'t any element of $_aTables';
				$this->_DRSprompt( $prompt, __LINE__ );
				continue;
			}
			$this->_tablesPostProcessFields( $TCATable );
		}
	}

	/**
	 * _tablesPostProcessFields( ) :
	 *
	 * @param string $table
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _tablesPostProcessFields( $table )
	{
		$aTCATableField = $this->_oTCA->getTableField();
		foreach( ( array ) $aTCATableField[ $table ] AS $field => $properties )
		{
			if( !is_array( $properties[ 'postProcess' ] ) )
			{
				continue;
			}
			switch( $properties[ 'postProcess' ][ 'key' ] )
			{
				case( 'foreign_table'):
					$this->_processForeignTable( $table, $field );
					break;
				case( 'internal_type'):
					$this->_processInternalType( $table, $field );
					break;
				default:
					$tableField = $table . '.' . $field;
					$header = 'ERROR: undefined key';
					$key = $properties[ 'postProcess' ][ 'key' ];
					$text = $tableField . ': Key for postProcess isn\'t defined! Key is "' . $key . '"';
					$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
					break;
			}
		}
	}

	/**
	 * _table( ) :
	 *
	 * @return void
	 * @access private
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _table()
	{
		//var_dump( __METHOD__, __LINE__, $this->_sCurrentTable );

		switch( TRUE )
		{
			case($this->_sCurrentTable == 'fe_users'):
				$this->_tableFeUsers();
				break;
			case($this->_sCurrentTable != 'fe_users'):
			default:
				$this->_tableMain();
				break;
		}
	}

	/**
	 * _tableFeUsers( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _tableFeUsers()
	{
		$action = $this->_getAction();
		//var_dump( __METHOD__, __LINE__, $sessionData, $uid );

		switch( $action )
		{
			case('add'):
			case('update'):
				// follow the workflow
				break;
			case('copy'):
			case('delete'):
			case('deleterecord'):
			case('index'):
				return;
			default:
				$this->_DRSswitchActionDie( $action, __METHOD__, __LINE__ );
				break;
		}

		switch( true )
		{
			case($this->_feuserIsLoggedIn()):
				$this->_tableMainUpdate();
				break;
			case(!$this->_feuserIsLoggedIn()):
			default:
				$this->_tableMainAdd();
				break;
		}
	}

	/**
	 * _tableMain( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _tableMain()
	{
		$action = $this->_getAction();
		//var_dump( __METHOD__, __LINE__, $sessionData, $uid );
		//var_dump( __METHOD__, __LINE__, $action );

		switch( $action )
		{
			case('add'):
				$this->_tableMainAdd();
				return;
			case('copy'):
				$this->_tableMainCopy();
				return;
			case('deleterecord'):
				$this->_tableMainDeleterecord();
				return;
			case('update'):
				$uid = $this->_getUid();
				$this->_tableMainUpdate( $uid );
				return;
			default:
				$this->_DRSswitchActionDie( $action, __METHOD__, __LINE__ );
				break;
		}
	}

	/**
	 * _tableMainAdd( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.8
	 * @since 7.4.0
	 */
	private function _tableMainAdd()
	{
		$data = $this->_oTables->getKeyValues();
		$table = $this->_sCurrentTable;
		// #i0271, 161120, dwildt, 1+
		$data = $this->_renderData( $data );
		$this->_aKeyValues[ $table ] = $data;

		$this->_SQLInsert( $table, $data, __LINE__ );

		$this->_setCurrentTableRecordUid();

		// #i0292, 170117, dwildt, 2+		
		$uid = $this->_sCurrentTableRecordUid[ $table ];
	}

	/**
	 * _tableMainCopy( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 8.0.21
	 */
	private function _tableMainCopy()
	{
		$table = $this->_sCurrentTable;
		$uidSrce = $this->_getUid( $table );
		$this->_SQLCopyMain( $table, $uidSrce, __LINE__, 0 );

		$uidDest = $this->_aTableSQLLastID[ $table ];
		$this->_tablesForeignCopy( $uidSrce, $uidDest );
	}

	/**
	 * _tableMainDeleterecord( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _tableMainDeleterecord()
	{
		$table = $this->_sCurrentTable;
		$ctrlDelete = $this->_getTCActrlDelete( $table );

		switch( true )
		{
			case($ctrlDelete):
				//var_dump( __METHOD__, __LINE__, $table, $ctrlDelete );
				$this->_tableMainDeleterecordHide( $ctrlDelete );
				break;
			case(!$ctrlDelete):
			default:
				$this->_tableMainDeleterecordDelete();
				$this->_tablesForeignDelete();
				break;
		}
	}

	/**
	 * _tableMainDeleterecordDelete( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _tableMainDeleterecordDelete()
	{
		$table = $this->_sCurrentTable;
		$uid = $this->_getUid();
		$where = 'uid =' . $uid;
		//var_dump( __METHOD__, __LINE__, $table, $uid );
		$this->_SQLDelete( $table, $where, __LINE__, 0 );
	}

	/**
	 * _tableMainDeleterecordHide( ) :
	 *
	 * @param	string 
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _tableMainDeleterecordHide( $ctrlDelete )
	{
		$table = $this->_sCurrentTable;
		$uid = $this->_getUid();
		$data = [
				$ctrlDelete => 1,
		];
		//var_dump( __METHOD__, __LINE__, $table, $uid, $ctrlDelete, $data );
		$this->_SQLUpdate( $table, $uid, $data, __LINE__, 0 );
	}

	/**
	 * _tableMainUpdate( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _tableMainUpdate( $uid = null )
	{
		$data = $this->_oTables->getKeyValues();
		// #i0296, 170122, dwildt, 1+
		$data = $this->_tablesSetUid( $data, $uid );
		$data = $this->_renderData( $data );

		$table = $this->_sCurrentTable;
		//var_dump( __METHOD__, __LINE__, $this->_sCurrentTable, $data );
		//die();
		$this->_aKeyValues[ $table ] = $data;

		switch( TRUE )
		{
			case( isset( $data[ 'uid' ] ) ):
				$uid = $data[ 'uid' ];
				unset( $data[ 'uid' ] );
				break;
			default:
				$table = 'Table: ' . $this->_sCurrentTable;
				$sUpdateData = 'Data: ' . implode( ',', $data );
				$sUpdateData = 'Data: ' . json_encode( $data );
				$header = 'SQL-ERROR: uid is missing';
				$text = 'Current data doesn\'t contain the uid! ' . $table . '. ' . $sUpdateData;
				$this->_zzDieWiPrompt( $header, $text, __METHOD__, __LINE__ );
				break;
		}

		$this->_SQLUpdate( $this->_sCurrentTable, $uid, $data, __LINE__ );

		$this->_setCurrentTableRecordUid();

		// #i0292, 170117, dwildt, 2+
		$uid = $this->_sCurrentTableRecordUid[ $table ];
	}

	/**
	 * _tablesSetUid( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.21
	 * @since 7.4.0
	 */
	private function _tablesSetUid( $data, $uid )
	{
		switch( true )
		{
			case(!empty( $data[ 'uid' ] )):
			case(empty( $uid )):
				return $data;
			default:
				$data[ 'uid' ] = $uid;
				return $data;
		}
	}

	/**
	 * _viewAssignAdd( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _viewAssignAdd()
	{
		// 170122, dwildt: If method is called from indexAction, it uses Templates > Index.html!
		$this->view->assignMultiple(
						array(
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelCancel' ),
								'buttonCancelClass' => 'btn btn-default button secondary expand',
								'header' => $this->_oPlugin->pi_getLL( 'actionAddHeader' ),
								'pid' => $this->settings[ 'constants' ][ 'interface' ][ 'page' ][ 'list' ],
								'promptHeader' => 'true',
						)
		);
	}

	/**
	 * _viewAssignCopy( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _viewAssignCopy()
	{
		// 170122, dwildt: If method is called from indexAction, it uses Templates > Index.html!
		$this->view->assignMultiple(
						array(
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelCancel' ),
								'buttonCancelClass' => 'btn btn-default button secondary expand',
								'class' => 'alert',
								'css' => $this->_powermailHideFormByCss(),
								'header' => $this->_oPlugin->pi_getLL( 'actionCopyHeader' ),
								'pid' => $this->settings[ 'constants' ][ 'interface' ][ 'page' ][ 'list' ],
								'prompt' => $this->_oPlugin->pi_getLL( 'actionCopyPrompt' ),
								'promptAlertBox' => 'true',
								'promptButtons' => 'true',
								'promptHeader' => 'true',
						)
		);
	}

	/**
	 * _viewAssignDelete( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _viewAssignDelete()
	{
		// 170122, dwildt: If method is called from indexAction, it uses Templates > Index.html!
		$showUid = $this->settings[ 'constants' ][ 'interface' ][ 'param' ][ 'showUid' ];
		$arguments = $this->request->getArguments();
		$this->view->assignMultiple(
						array(
								'arguments' => [
										$showUid => ( int ) $arguments[ $showUid ],
								],
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelCancel' ),
								'buttonCancelClass' => 'btn btn-default button secondary expand',
								'class' => 'alert',
								'css' => $this->_powermailHideFormByCss(),
								'header' => $this->_oPlugin->pi_getLL( 'actionDeleteHeader' ),
								'pid' => $this->settings[ 'constants' ][ 'interface' ][ 'page' ][ 'list' ],
								'prompt' => $this->_oPlugin->pi_getLL( 'actionDeletePrompt' ),
								'promptAlertBox' => 'true',
								'promptButtonDelete' => 'true',
								'promptButtons' => 'true',
								'promptHeader' => 'true',
						)
		);
	}

	/**
	 * _viewAssignIndex( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _viewAssignIndex()
	{
		$this->view->assignMultiple(
						array(
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelBack' ),
								'buttonCancelClass' => 'btn btn-default button secondary expand',
								'class' => 'alert',
								'css' => $this->_powermailHideFormByCss(),
								'header' => $this->_oPlugin->pi_getLL( 'errorNoActionHeader' ),
								'pid' => $this->settings[ 'constants' ][ 'interface' ][ 'page' ][ 'list' ],
								'prompt' => ''
								. '<p>'
								. $this->_oPlugin->pi_getLL( 'errorNoActionPrompt' )
								. '<br />'
								. $this->_oPlugin->pi_getLL( 'promptBrowser' )
								. '<br />'
								. __METHOD__ . '#' . __LINE__
								. '<br />'
								. '<br />'
								. $this->_oPlugin->pi_getLL( 'promptSorry' )
								. '</p>'
								,
								'promptAlertBox' => 'true',
								'promptButtons' => 'true',
								'promptHeader' => 'true',
						)
		);
	}

	/**
	 * _viewAssignUpdate( ) :
	 *
	 * @return void
	 * @access private
	 * @version 8.0.18
	 * @since 8.0.18
	 */
	private function _viewAssignUpdate()
	{
		// 170122, dwildt: If method is called from indexAction, it uses Templates > Index.html!
		$this->view->assignMultiple(
						array(
								'buttonCancel' => $this->_oPlugin->pi_getLL( 'labelCancel' ),
								'buttonCancelClass' => 'btn btn-default button secondary expand',
								'header' => $this->_oPlugin->pi_getLL( 'actionUpdateHeader' ),
								'pid' => $this->settings[ 'constants' ][ 'interface' ][ 'page' ][ 'list' ],
								'promptHeader' => 'true',
						)
		);
//		$this->view->assignMultiple(
//						array(
//								'class' => 'alert',
//								'css' => null,
//								//						'css' => $this->_powermailHideFormByCss(),
//								'header' => $this->_oPlugin->pi_getLL( 'actionUpdateHeader' ),
//								'pid' => $this->settings[ 'constants' ][ 'interface' ][ 'page' ][ 'list' ],
//								'prompt' => ''
//								. '<p>'
//								. $this->_oPlugin->pi_getLL( 'actionUpdatePrompt' )
//								. '<br />'
//								. $this->_oPlugin->pi_getLL( 'promptBrowser' )
//								. '<br />'
//								. __METHOD__ . '#' . __LINE__
//								. '<br />'
//								. '<br />'
//								. $this->_oPlugin->pi_getLL( 'promptSorry' )
//								. '</p>'
//						,
//								'promptAlertBox' => 'true',
//						)
//		);
	}

	/**
	 * _zzDieWiPrompt( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 7.4.0
	 * @since 7.4.0
	 */
	private function _zzDieWiPrompt( $header, $text, $method, $line )
	{
		$prompt = '
      <h1 style="color:red;">
        ' . $header . '
      </h1>
      ' . $text . '
      <p>
        Sorry for the trouble. Browser - TYPO3 without PHP<br />
        Error occures here: ' . $method . ' at #' . $line . '
      </p>
      <p>
        If you need any help, please visit the
        <a href="http://typo3-browser-forum.de/" target="_blank" title="TYPO3 Browser Forums. 500 TYPO3-integrators are registered.">
          TYPO3 Browser Forum &raquo;</a>
      </p>
      ';
		die( $prompt );
	}

	/**
	 * _zzGetMaxDbUid( ) :
	 *
	 * @param string $table
	 * @return integer  $uid
	 * @access private
	 * @internal #i0271
	 * @version 8.0.8
	 * @since 8.0.0
	 */
	private function _zzGetMaxDbUid( $table )
	{
		$maxUid = false;

		$select_fields = 'max(`uid`) AS "uid"';
		$from_table = '`' . $table . '`';
		$where_clause = '';
		$groupBy = '';
		$orderBy = '';
		$limit = '';
		$uidIndexField = '';

		//var_dump($GLOBALS['TYPO3_DB']->SELECTquery($select_fields, $from_table, $where_clause, $groupBy, $orderBy, $limit, $uidIndexField));
		$rows = $GLOBALS[ 'TYPO3_DB' ]->exec_SELECTgetRows( $select_fields, $from_table, $where_clause, $groupBy, $orderBy, $limit, $uidIndexField );

		if( is_array( $rows ) && count( $rows ) > 0 )
		{
			$maxUid = $rows[ 0 ][ 'uid' ];
		}
		return $maxUid;
	}

}
