<?php

namespace Netzmacher\Browser\UserFunc;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2018 - Dirk Wildt <http://wildt.at.die-netzmacher.de>
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * The class tx_browser_pi1_session bundles methods for the session management
 *
 * @author    Dirk Wildt <http://wildt.at.die-netzmacher.de>
 * @package    TYPO3
 * @subpackage  browser
 *
 * @version 9.1.1
 * @since 9.1.1
 */
class Session
{
	
	private $_conf;

	/**
	 * _SQLError( ) :
	 *
	 * @param string $query
	 * @param integer $line
	 * @return void
	 * @access private
	 * @version 9.1.1
	 * @since 9.1.1
	 */
	private function _SQLError( $query, $method = __METHOD__, $line = __LINE__ )
	{
		$sqlError = $GLOBALS[ 'TYPO3_DB' ]->sql_error();
		if( empty( $sqlError ) )
		{
			return;
		}

		$sqlErrno = $GLOBALS[ 'TYPO3_DB' ]->sql_errno();
		$header = 'SQL-ERROR';
		$text = ''
						. '<p>'
						. '  Error: ' . $sqlError . '<br />'
						. '  Error number: ' . $sqlErrno . '<br />'
						. '  Query: ' . $query . '<br />'
						. '</p>'
		;
		$this->_zzDieWiPrompt( $header, $text, $method, $line );
	}

	/**
	 * _clearCache( ) :
	 *
	 * @return void
	 * @access private
	 * @internal #i0411
	 * @version 9.1.1
	 * @since 9.1.1
	 */
	private function _clearCache()
	{
		$query = 'TRUNCATE `cf_cache_pages`';
		$GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$this->_SQLError( $query, __METHOD__, __LINE__ );

		$query = 'TRUNCATE `cf_cache_pagesection`';
		$GLOBALS[ 'TYPO3_DB' ]->sql_query( $query );
		$this->_SQLError( $query, __METHOD__, __LINE__ );

//	Code from below is proper only, if an admin is logged in in the backend.
//		$tce = GeneralUtility::makeInstance( DataHandler::class );
//		$tce->stripslashes_values = false;
//		$tce->admin = 1;
//		$tce->start( [ ], [ ] );
//		$tce->clear_cacheCmd( $cacheCmd );
	}

	/**
	 * _cObj( ) :
	 *
	 * @return string
	 * @access private
	 * @version 9.1.1
	 * @since 9.1.1
	 */
	private function _cObj( $property )
	{
		if( !isset( $this->_conf[ $property . '.' ] ) )
		{
			return $this->_conf[ $property ];
		}
		
		$cObj_name = $this->_conf[ $property ];
		$cObj_conf = $this->_conf[ $property . '.'];

		return $this->cObj->cObjGetSingle( $cObj_name, $cObj_conf );
	}

	/**
	 * _getNameOfDataSpace( ): Get the name of the session data space.
	 *                        The name is user, if a frontend user is logged in.
	 *                        The name is ses, if any frontend user isn't logged in.
	 *
	 * @return	string		user || ses
	 * @version 9.1.0
	 * @since 9.1.0
	 */
	private function _getNameOfDataSpace()
	{
		switch( $GLOBALS[ 'TSFE' ]->loginUser )
		{
			case( true ):
				return 'user';
			default:
				return 'ses';
		}
	}

	/**
	 * _zzDieWiPrompt( ) :
	 *
	 * @access private
	 * @return boolean
	 * @version 9.1.1
	 * @since 9.1.1
	 */
	private function _zzDieWiPrompt( $header, $text, $method, $line )
	{
		$prompt = '
      <h1 style="color:red;">
        ' . $header . '
      </h1>
      ' . $text . '
      <p>
        Sorry for the trouble. Browser - TYPO3 without PHP<br />
        Error occures here: ' . $method . ' at #' . $line . '
      </p>
      <p>
        If you need any help, please visit the
        <a href="http://typo3-browser-forum.de/" target="_blank" title="TYPO3 Browser Forums. 500 TYPO3-integrators are registered.">
          TYPO3 Browser Forum &raquo;</a>
      </p>
      ';
		die( $prompt );
	}

	/**
	 * setShowUidBy():
	 *
	 * @param  string          Empty string (no content to process)
	 * @param  array           TypoScript configuration
	 * @return string          void
	 * @internal #i0405
	 * @version 9.1.1
	 * @since 9.1.0
	 */
	public function setShowUid( $content, $conf )
	{
		unset( $content );
		$this->_conf = $conf;
		$space = $this->_getNameOfDataSpace();
		$table = $this->_cObj( 'table' );
		$uid = $this->_cObj( 'uid' );
		$dontClearCache = $this->_cObj( 'dontClearCache' );

		if( empty( $uid ) )
		{
			return;
		}

		// #i0411, 180221, dwildt, 4+
		switch( true )
		{
			case( $dontClearCache == 1 ):
			case( $dontClearCache == true ):
				// do noting;
				// #i0431, 180501, dwildt, 1+
				break;
			case(empty( $dontClearCache )):
			case( $dontClearCache == 0 ):
			case( $dontClearCache == false ):
			default:
				$this->_clearCache();
		}

		$arrSessionData = [
				$table => [
						'lastSelected' => $uid,
				],
		];

		//var_dump( __METHOD__, __LINE__, $arrSessionData, $space );

		$GLOBALS[ 'TSFE' ]->fe_user->setKey( $space, 'tx_browser_pi1', $arrSessionData );
		$GLOBALS[ "TSFE" ]->storeSessionData();

		return;
	}

}
